require "rest-client"
require "base64"
require "nokogiri"
require "addressable/uri"

module Hermes
  class RexClient::Client
    class ClientError < StandardError
      attr_reader :response

      def initialize(response)
        super(message)
        @response = response
        if false
          begin
            message = "#{error_message} - CODE '#{code}'"
          rescue => e
            message = "Error while call API, Unparsable response (error: #{e.inspect}) : #{self.response.to_s.force_encoding("UTF-8")}"
          end
        end
        message = "Error : #{response.to_s.force_encoding("UTF-8")}"

        super(message)
      end

      def request
        response.request
      end
    end

    class Forbidden < StandardError
    end

    class NotFound < StandardError
    end

    attr_accessor :auth_url
    attr_accessor :base_url
    attr_accessor :ed_entity
    attr_accessor :ed_lessor
    attr_accessor :ed_contact
    attr_accessor :base_url
    attr_accessor :client_id
    attr_accessor :client_secret
    attr_accessor :username
    attr_accessor :password
    attr_accessor :logger
    attr_accessor :token

    def initialize(attrs = {})
      @auth_url = attrs.fetch(:base_url, RexClient.config.auth_url)
      @base_url = attrs.fetch(:base_url, RexClient.config.base_url)
      @ed_entity = attrs.fetch(:ed_entity, RexClient.config.ed_entity)
      @ed_lessor = attrs.fetch(:ed_lessor, RexClient.config.ed_lessor)
      @ed_contact = attrs.fetch(:ed_lessor, RexClient.config.ed_contact)
      @client_id = attrs.fetch(:base_url, RexClient.config.client_id)
      @client_secret = attrs.fetch(:base_url, RexClient.config.client_secret)
      @username = attrs.fetch(:username, RexClient.config.username)
      @password = attrs.fetch(:password, RexClient.config.password)
      @logger = attrs.fetch(:logger, RexClient.config.logger)
    end

    def log_error(error, classname = nil, id = nil)
      return unless classname && id

      instance = classname.constantize.find(id)
      return unless instance
      instance.add_error(error)
    end

    def log_success(classname = nil, id = nil)
      return unless classname && id

      instance = classname.constantize.find(id)
      return unless instance
      instance.set(last_send_has_error: false, updated_at: DateTime.current)
    end

    def get_token
      @token = RestClient.post(
        @auth_url,
        headers: { "Content-Type" => "application/x-www-form-urlencoded" },
        grant_type: "password",
        client_id: @client_id,
        client_secret: @client_secret,
        username: @username,
        password: @password,
        log: logger,
      ).json_to_h[:access_token]
    end

    def request_header
      {
        "Content-Type" => "application/x-www-form-urlencoded",
        "Authorization" => "Bearer #{@token}",
      }
    end

    def get_structure_ed(type)
      return @ed_entity if type == "entity"
      return @ed_lessor if type == "lessor"
      return @ed_contact if type == "contact"
      return
    end

    def find_base_structure_by_id(id, type = "entity")
      get_token if !@token

      begin
        return RestClient.get(
          "#{@base_url}/entityDefinition/#{get_structure_ed(type)}/entity/#{id}",
          request_header,
        ).json_to_h
      rescue RestClient::ExceptionWithResponse => e
        return;
      end
    end

    def find_entity_by_id(id)
      find_base_structure_by_id(id, "entity")
    end

    def find_lessor_by_id(id)
      find_base_structure_by_id(id, "lessor")
    end

    def find_structure_by_id(id)
      find_base_structure_by_id(id, "entity") || find_base_structure_by_id(id, "lessor")
    end

    def search_base(opt, type = "entity", page = 1, previous_result = [], total = nil)
      if total && total <= previous_result.length
        return previous_result
      end
      
      get_token if !@token
      
      size = 10
      from = (page - 1) * size

      query = {
        from: from,
        size: size,
        sortField: "fields.id_cristal",
      }.merge(opt).to_query

      r = RestClient.get(
        "#{@base_url}/entityDefinition/#{get_structure_ed(type)}/entity?#{query}",
        request_header,
      ).json_to_h
      
      search_base(opt, type, page + 1, previous_result + r[:items], r[:total])
    end

    def search_entities(opt = {})
      search_base(opt, "entity")
    end

    def search_lessors(opt = {})
      search_base(opt, "lessor")
    end

    def search_contacts(opt = {})
      search_base(opt, "contact")
    end

    def search_structures(opt = {})
      search_base(opt, "entity") + search_base(opt, "lessor")
    end
  end
end
