
require 'rails_helper'
require 'vcr_config'
#require 'json'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

  MUTATE_PARAMS = {
    title: 'String',
    #siret: 'String',
    #siren: 'String',
    automated_request_validations: 'Boolean',
    delegated_validations: 'Boolean',
    priorities_enabled: 'Boolean',
    in_peec: 'Boolean',
    achieved_goal: 'Boolean',
    spent_on_completed_requests: 'Integer',
    spent_on_ongoing_requests: 'Integer',
    spent_on_other: 'Integer',
    annual_stock: 'Integer',
    auto_validation: "Boolean"
  }

  ACCEPTABLE_PARAMS = %w[
    als_id
    title
    siret
    siren
    is_root
    root_id
    automated_request_validations
    delegated_validations
    priorities_enabled
    in_peec
    achieved_goal
    spent_on_completed_requests
    spent_on_ongoing_requests
    spent_on_other
    annual_stock
    total_spent
    available_points
    applicant_ids
    lessor_id
    group_id
    supervisor_group_id
    manager_group_id
    auto_validation
  ]

  VALID_OPERATORS = %w[
    $in
    $nin
    $ne
    $gt
    $gte
    $lt
    $lte
    $eq
    $regex
    $all
  ]

  HERMES_CONFIG['identity_provider_key'] = 'als_hermes_salarie'

  describe Api::Dmo::EntitiesController, :type => :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    let!(:responsible) {create(:user, roles: ['Administrator'])}
    let!(:superadministrateur) {create(:user, auth_roles: ['superadministrateur'])}
    let!(:user) {create(:user)}
    let!(:identity_provider) {create(:identity_provider, 
      key: 'als_hermes_salarie', 
      create_user_url: 'https://api-rcc.almerys.com/als-back/accounts', 
      create_user_headers: {
        'Content-Type': 'application/json',
        'X-GeXRT-API-Key': 'fbb003ae-4de4-4f0e-834f-7b509d170609',
      }
      )}
    

    describe 'POST /entities' do
      describe 'Create entity with minimal valid datas' do
        before :each do
          authenticated_header(request, responsible)
          post 'create', {
              data: {
                  attributes: {
                  }
              }
          }
        end
        it 'returns status code 201' do
          expect(response).to have_http_status(201)
        end
      end
      MUTATE_PARAMS.each do |param, type|
        describe 'post request with ' + param.to_s + ' (' + type.to_s + ') is valid' do
          let(:params) do
            attr = {}
            attr[param] = get_typed_value(type)
            attr
          end
          before :each do
            authenticated_header(request, responsible)
            post 'create', {
                data: {
                    attributes: params
                }
            }
          end
          it 'returns status code 201 and the value appears in json' do
            expect(response).to have_http_status(201)
            if (type == 'Date' || type == 'DateTime')
              expect(json['data']['attributes'][param.to_s]).not_to be_empty
              expect(json['data']['attributes'][param.to_s]).not_to eq('')
            else
              expect(json['data']['attributes'][param.to_s]).to eq(get_typed_value(type))
            end
          end
        end
      end
      #applicant_ids: 'Array',
      describe 'post request with applicant_ids' do
        let!(:applicant2) {create(:applicant)}
        let!(:applicant) {create(:applicant)}
        let!(:applicant2) {create(:applicant)}
        before :each do
          authenticated_header(request, responsible)
          post 'create', {
              data: {
                  attributes: {
                      applicant_ids: [applicant.id.to_s, applicant2.id.to_s]
                  }
              }
          }
        end
        it 'returns status code 201 and the value appears in json' do
          expect(response).to have_http_status(201)
          #TODO on pourrait s'attendre a retrouver les ids des housings requests mais c'est coherent car elles sont deja en belongs to avec un autre applicant
          expect(json['data']['attributes']['applicant_ids']).to eq([])
        end
      end
      # TODO: housing_offer_ids: 'Array',
      # describe 'post request with housing_offer_ids' do
      #   let!(:applicant2) {create(:applicant)}
      #   let!(:housing_offer) {create(:housing_offer)}
      #   let!(:housing_offer2) {create(:housing_offer)}
      #   before :each do
      #     authenticated_header(request, responsible)
      #     post 'create', {
      #         data: {
      #             attributes: {
      #                 housing_offer_ids: [housing_offer.id.to_s, housing_offer2.id.to_s]
      #             }
      #         }
      #     }
      #   end
      #   it 'returns status code 201 and the value appears in json' do
      #     expect(response).to have_http_status(201)
      #     #TODO on pourrait s'attendre a retrouver les ids des housings requests mais c'est coherent car elles sont deja en belongs to avec un autre applicant
      #     expect(json['data']['attributes']['housing_offer_ids']).to eq([])
      #   end
      # end
      #lessor_id: 'String'
      describe 'post request with lessor_id is valid' do
        let!(:lessor) {create(:lessor, user_id: responsible.id, title: 'Lessor')}
        before :each do
          authenticated_header(request, responsible)
          post 'create', {
              data: {
                  attributes: {lessor_id: lessor.id.to_s}
              }
          }
        end
        it 'returns status code 201 and the value appears in json' do
          expect(response).to have_http_status(201)
          expect(json['data']['attributes']['lessor_id']).to eq(lessor.id.to_s)
        end
      end
    end

    describe 'GET /entities' do
      let!(:entity1) {create(:entity)}
      let!(:entity3) {create(:entity)}
      let!(:entity2) {create(:entity)}

      [
        {user: 'responsible', total_objects: 3},
        {user: 'superadministrateur', total_objects: 3},
        {user: 'user', total_objects: 0},
      ].each do |a| 
        describe "as #{a[:user]}" do
          before :each do
            authenticated_header(request, eval(a[:user]))
            get 'index'
          end
          it 'returns status code 200' do
            expect(response).to have_http_status(200)
          end
          it 'returns a valid json' do
            expect(json).not_to be_empty
            expect(json['data'].length).to eq(a[:total_objects])
            expect(json['meta']['pagination']).to be_present
            expect(json['meta']['pagination']['total_objects']).to eq(a[:total_objects])
          end
        end
      end
      
      describe 'GET /entities with batch of ids' do
        let!(:entity1) {create(:entity)}
        let!(:entity3) {create(:entity)}
        let!(:entity2) {create(:entity)}
        before :each do
          authenticated_header(request, responsible)
          get 'index', {ids: [entity1.id.to_s, entity2.id.to_s]}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data'].length).to eq(2)
        end
      end

      ACCEPTABLE_PARAMS.each do |acceptable_param|
        type = MUTATE_PARAMS[acceptable_param.to_sym] || MUTATE_PARAMS[acceptable_param.to_s]
        if type
          describe 'GET /entities with sort on field ' + acceptable_param + ' works ' do
            let!(:entity3) {create(:entity, acceptable_param => get_typed_value(type, 2))}
            let!(:entity1) {create(:entity, acceptable_param => get_typed_value(type, 0))}
            let!(:entity2) {create(:entity, acceptable_param => get_typed_value(type, 1))}
            before :each do
              authenticated_header(request, responsible)
              get 'index', {sort: {acceptable_param => 1}}
            end
            it 'returns sorted results' do
              expect(response.status).to eq(200)
              if type == 'DateTime'
                entity_attribute = json['data'].map {|unit| DateTime.parse(unit['attributes'][acceptable_param])}
                expect(entity_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
              elsif type == 'Date'
                entity_attribute = json['data'].map {|unit| Date.parse(unit['attributes'][acceptable_param])}
                expect(entity_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
              else
                entity_attribute = json['data'].map {|unit| unit['attributes'][acceptable_param]}
                expect(entity_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
              end
            end
          end
        end
      end

      describe 'GET /entities with operators' do
        let!(:entity1) {create(:entity, siret: 'Super000000000', spent_on_completed_requests: 50)}
        let!(:entity3) {create(:entity, siret: 'Super000000000', spent_on_completed_requests: 150)}
        let!(:entity2) {create(:entity, siret: 'Auper000000001', spent_on_completed_requests: 150)}
        describe 'with $lt' do
          before :each do
            authenticated_header(request, responsible)
            get 'index', {spent_on_completed_requests: {'$lt': 100}}
          end
          it 'returns results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(1)
          end
        end
        describe 'with $in' do
          before :each do
            authenticated_header(request, responsible)
            get 'index', {siret: {'$in': ['Auper000000001', 'flamand']}}
          end
          it 'returns results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(1)
          end
        end
        describe 'with $ne' do
          before :each do
            authenticated_header(request, responsible)
            get 'index', {spent_on_completed_requests: {'$ne': 50}}
          end
          it 'returns sorted results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(2)
          end
        end
        describe 'with $regex' do
          before :each do
            authenticated_header(request, responsible)
            get 'index', {siret: {'$regex': /^S/}}
          end
          it 'returns sorted results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(2)
          end
        end
      end
    end

    describe 'GET /entities/:id' do
      describe 'verif it work' do
        let!(:entity1) {create(:entity)}
        before :each do
          authenticated_header(request, responsible)
          get 'show', {id: entity1.id}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
        end
      end
      describe 'verif it work not work with a non authorized user' do
        let!(:entity1) {create(:entity)}

        before :each do
          authenticated_header(request, user)
          get 'show', {id: entity1.id}
        end
        it 'returns status code 404' do
          expect(response).to have_http_status(404)
        end
      end
    end

    describe 'PATCH /entities/:id' do
      entity_params = {}
      MUTATE_PARAMS.each do |param, type|
        entity_params[param] = get_typed_value(type)
      end

      let!(:entity) {create(:entity, **entity_params)}
      MUTATE_PARAMS.each do |param, type|
        describe 'patch request with ' + param.to_s + ' ' + type.to_s + 'is valid' do
          let(:params) do
            attr = {}
            attr[param] = get_typed_value(type, 2)
            attr
          end
          before :each do

            authenticated_header(request, responsible)
            patch 'update', {
                id: entity.id.to_s,
                data: {
                    attributes: params
                }
            }
          end
          it 'returns status code 200 and the value appears in json' do
            expect(response).to have_http_status(200)
            if (type == 'Date' || type == 'DateTime')

              expect(json['data']['attributes'][param.to_s]).not_to be_empty
              expect(json['data']['attributes'][param.to_s]).not_to eq('')
            else
              expect(json['data']['attributes'][param.to_s]).to eq(get_typed_value(type, 2))
            end
          end
        end
      end

      describe 'DELETE /entities/:id' do
        let!(:entity) {create(:entity, user_id: responsible.id.to_s)}
        before :each do
          authenticated_header(request, responsible)

          delete 'destroy', {
              id: entity.id.to_s
          }
        end
        it 'returns status code 404' do
          expect(response).to have_http_status(204)
        end
      end
    end

    context 'Superadmin searches' do
      let!(:superadmin) {create(:user, roles: ['Superadmin'], first_name: "test_first", last_name: "test_last")}
      let!(:entity) {create(:entity, title: "test", siret: "12345671234567",  siren: "123456789",id_cristal: "1")}

      describe 'GET /entities/admin_search' do
        describe "search with user that is not a Superadmin" do
          before do
            authenticated_header(request, user)
            get 'admin_search'
          end
      
          it "refuses the access" do
            body = JSON.parse(response.body)
            expect(body["code"]).to eq(401)
          end
        end
  
        describe "search with no params at all" do
          before do
            authenticated_header(request, superadmin)
            get 'admin_search'
          end
      
          it "does give an error" do
            body = JSON.parse(response.body)
            expect(body["code"]).to eq(401)
          end
        end
  
        describe "search with wrong params" do
          before do
            authenticated_header(request, superadmin)
            get 'admin_search', {"test" => "test1"}
          end
      
          it "does give an empty hash" do
            body = JSON.parse(response.body)
            expect(body["code"]).to eq(401)
          end
        end
        
        describe "search with good params" do

          before do
            authenticated_header(request, superadmin)
            get 'admin_search', {"entity_params" => "{\"id\":\"#{entity.id.to_s}\"}"}
          end
      
          it "does give a housing request" do
            body = JSON.parse(response.body)
            # Params from entites and users 
            expect(body["message"]["entities_and_users"]).to eq([{
              "entity" => {
                "id"=> entity.id.to_s,
                "siret"=>entity.siret,
                "siren"=> entity.siren,
                "id_cristal"=>entity.id_cristal,
                "title"=>entity.title,
              },
              "user"=>[]}])
          end
        end
        describe "search with good params + user params" do

          before do
            entity.user = superadmin
            entity.save
            authenticated_header(request, superadmin)
            get 'admin_search', {"entity_params" => "{\"id\":\"#{entity.id.to_s}\"}",
                                 "user_params" => "{\"id\":\"#{entity.user.id.to_s}\"}"}
          end
      
          it "does give a housing request" do
            body = JSON.parse(response.body)
            # Params from entites and users 
            expect(body["message"]["entities_and_users"]).to eq([{
              "entity" => {
                "id"=> entity.id.to_s,
                "siret"=>entity.siret,
                "siren"=> entity.siren,
                "id_cristal"=>entity.id_cristal,
                "title"=>entity.title,
              },
              "user"=>[
                {"auth_roles"=>nil,
                  "first_name"=> superadmin.first_name,
                  "id"=>superadmin.id.to_s,
                  "last_name"=>superadmin.last_name}
              ]}])
          end
        end
        
      end 
  
    end 
    [
      {
        route_name: 'create_applicant',
        user_info_successfull:{
          last_name: 'Demandeur', 
          login: 'test.demandeur2',
        },
        user_info_user_error: {
          last_name: 'Demandeur', 
          login: 'test.demandeur3',
        },
      },{
        route_name: 'create_user',
        user_info_successfull:{
          last_name: 'Collaborateur', 
          login: 'test.collacorateur',
        },
        user_info_user_error: {
          last_name: 'Demandeur', 
          login: 'test.collacorateur2',
        },
      },{
        route_name: 'create_supervisor',
        user_info_successfull:{
          last_name: 'Supervisor', 
          login: 'test.supervisor',
        },
        user_info_user_error: {
          last_name: 'Supervisor', 
          login: 'test.supervisor2',
        },
      },
    ].each do |route|
      describe "POST /entities/:id/#{route[:route_name]}" do
        let!(:entity) {create(:entity)}
        let!(:collaborateur_entreprise) {create(:user, auth_roles: ['collaborateur entreprise'], group_ids:[entity.group_id])}
        let!(:supervisor) {create(:user, auth_roles: ['entreprise'], group_ids:[entity.supervisor_group_id])}
        let!(:entity_admin) {create(:user, auth_roles: ['entreprise administrateur'], group_ids:[entity.manager_group_id])}
        let!(:random_user) {create(:user, email: 'random_user@mail.com')}

        describe 'Entity not found' do
          before :each do
            authenticated_header(request, supervisor)
            post route[:route_name], {id: 'notfound'}
          end

          it 'returns status code 404' do
            expect(response).to have_http_status(404)
          end
        end

        describe 'user cannot modify' do
          before :each do
            authenticated_header(request, collaborateur_entreprise)
            post route[:route_name], {id: entity.id}
          end

          it 'returns status code 401' do
            expect(response).to have_http_status(401)
          end
        end

        describe "missing params to #{route[:route_name]}" do
          before :each do
            authenticated_header(request, supervisor)
            post route[:route_name], {id: entity.id}
          end

          it 'returns status code 422' do
            expect(response).to have_http_status(422)
          end
        end

        describe 'successfull' do
          before :each do
            authenticated_header(request, supervisor)
            VCR.use_cassette("entity_#{route[:route_name]}_user", :match_requests_on => [:method, :uri, :path]) do
              data = {
                id: entity.id,
                first_name: 'Test', 
                last_name: route[:user_info_successfull][:last_name], 
                email: "#{route[:user_info_successfull][:login]}@mail.com",
                login: route[:user_info_successfull][:login],
                gender: 'M',
                phone_number: '1234567890',
              }

              data = data.merge({nationality: 'Française'}) if route[:route_name] == "create_applicant"
              post route[:route_name], data
            end
          end
          
          it 'returns status code 201' do
            user = User.find_by(login: route[:user_info_successfull][:login])
            expect(response).to have_http_status(201)
            expect(user).not_to be_blank
            if route[:route_name] == "create_applicant"
              expect(json['data']['attributes']['last_name']).to eq(route[:user_info_successfull][:last_name])
              expect(json['data']['attributes']['main_user_id']).not_to be_nil
              expect(json['data']['attributes']['nationality']).to eq(Hermes::Applicant.find(json['data']['id']).nationality)
              expect(json['data']['attributes']['entity_id']).to eq(entity.id.to_s)

            else
              expect(json['data']['attributes']['last_name']).to eq(route[:user_info_successfull][:last_name])
              expect(json['data']['attributes']['login']).not_to be_nil
              expect(Hermes::Entity.readable_by(user).find_by(id: entity.id)).not_to be_nil
            end

            login = route[:user_info_successfull][:login]
            case route[:route_name]
            when 'create_applicant'
              expect(User.writable_by(collaborateur_entreprise).find_by(login: login)).not_to be_nil
              expect(User.writable_by(supervisor).find_by(login: login)).not_to be_nil
              expect(User.writable_by(entity_admin).find_by(login: login)).not_to be_nil
            when 'create_user'
              expect(Hermes::Entity.readable_by(user).find_by(id: entity.id)).not_to be_nil
              expect(User.writable_by(collaborateur_entreprise).find_by(login: login)).to be_nil
              expect(User.writable_by(supervisor).find_by(login: login)).not_to be_nil
              expect(User.writable_by(entity_admin).find_by(login: login)).not_to be_nil
            when 'create_supervisor'
              expect(Hermes::Entity.readable_by(user).find_by(id: entity.id)).not_to be_nil
              expect(User.writable_by(collaborateur_entreprise).find_by(login: login)).to be_nil
              expect(User.writable_by(supervisor).find_by(login: login)).to be_nil
              expect(User.writable_by(entity_admin).find_by(login: login)).not_to be_nil
            end
          end
        end

        describe 'beiam user already exist' do
          before :each do
            authenticated_header(request, supervisor)
            VCR.use_cassette("entity_#{route[:route_name]}_user_already_exist", :match_requests_on => [:method, :uri, :path]) do
              post route[:route_name], {
                id: entity.id,
                first_name: 'Test', 
                last_name: route[:user_info_successfull][:last_name], 
                email: "#{route[:user_info_successfull][:login]}@mail.com",
                login: route[:user_info_successfull][:login],
                gender: 'M',
                phone_number: '1234567890',
              }
            end
          end
          
          it 'returns status code 201' do
            expect(response).to have_http_status(201)
            if route[:route_name] == "create_applicant"
              expect(json['data']['attributes']['last_name']).to eq(route[:user_info_successfull][:last_name])
              expect(json['data']['attributes']['main_user_id']).not_to be_blank
              expect(json['data']['attributes']['entity_id']).to eq(entity.id.to_s)
            else
              expect(json['data']['attributes']['last_name']).to eq(route[:user_info_successfull][:last_name])
              expect(json['data']['attributes']['login']).not_to be_blank
            end
            expect(User.find_by(login: route[:user_info_successfull][:login])).not_to be_blank
          end
        end

        describe 'beiam user error' do
          before :each do
            authenticated_header(request, supervisor)
            VCR.use_cassette("entity_#{route[:route_name]}_user_error", :match_requests_on => [:method, :uri, :path]) do
              post route[:route_name], {
                id: entity.id,
                first_name: 'Test', 
                last_name: route[:user_info_user_error][:last_name], 
                email: "#{route[:user_info_user_error][:login]}@mail.com",
                login: route[:user_info_user_error][:login],
                gender: 'error',
                phone_number: '1234567890',
              }
            end
          end
          
          it 'returns status code 400' do
            expect(response).to have_http_status(400)
            expect(User.find_by(email: "#{route[:user_info_user_error][:login]}@mail.com")).to eq(nil)
          end
        end

        describe 'email user already exist' do
          before :each do
            authenticated_header(request, supervisor)
            VCR.use_cassette("entity_#{route[:route_name]}_email_user_exist", :match_requests_on => [:method, :uri, :path]) do
              post route[:route_name], {
                id: entity.id,
                first_name: 'Test', 
                last_name: Random, 
                email: "random_user@mail.com",
                login: "random_user",
                gender: 'M',
                phone_number: '1234567890',
              }
            end
          end
          it 'returns status code 400' do
            expect(response).to have_http_status(400)
            expect(User.find_by(email: "random_user@mail.com").login).to eq(nil)
          end
        end

      end
    end

    describe 'PUT /entities/:id/switch_manual_mode' do
      let!(:entity) {create(:entity)}
      let!(:admin) { create :user, roles: ['Superadmin']}
      before do 
        authenticated_header(request, admin)

        put 'switch_manual_mode',{
          id: entity.id 
        }
      end 

      it 'does switch manual mode' do 
        resp = JSON.parse(response.body)
        puts resp 
        expect(resp['code']).to eq(200)
      end 
    end 
  end

  describe Api::Dmo::EntitiesController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
    require_relative '../../swagger_helper'
    TAGS_API_DMO_ENTITIES = 'API_DMO_ENTITIES'
    include JwtAuthHelper
    extend HermesSwaggerJsonParams
    include SwaggerJsonAttributes
    let!(:responsible) {create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator'])}
    let!(:user) {create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com')}

    context 'as superadmin' do 

      let!(:superadmin) {create(:user, roles: ['Superadmin'])}

      let!(:main_applicant) {
        create(
          :applicant,
          user: superadmin,
          monthly_charges: {},
          nationality: "3"
        )
      }

      let!(:applicant_id) {
        create(
          :applicant,
          user: superadmin,
          monthly_charges: {},
          nationality: "2"
        )
      }
      let!(:entity) {create(:entity, title: "test", siret: "12345671234567", id_cristal: "1")}

      let!(:housing_request) {
        create(
          :housing_request,
          main_applicant: main_applicant,
          applicant_ids: [applicant_id.id.to_s])
        
      }

      path '/api/dmo/entities/admin_search' do
        get 'get users and entities from searches params ' do
          tags TAGS_API_DMO_ENTITIES
          produces 'application/json'
          consumes 'application/json'

          security [Bearer: {}]
          json_params = {}
          json_params[:entity_params] = {}
          json_params[:entity_params][:id] = {type: :string}
          json_params[:entity_params][:id_cristal] = {type: :string}
          json_params[:entity_params][:siren] = {type: :string}
          json_params[:entity_params][:siret] = {type: :string}
          json_params[:entity_params][:title] = {type: :string}
          json_params[:user_params] = {}
          json_params[:user_params][:id] = {type: :string}
          json_params[:user_params][:first_name] = {type: :string}
          json_params[:user_params][:last_name] = {type: :string}
          json_params[:user_params][:auth_roles] = {type: :string}
         
          parameter name: :params, in: :body, schema: json_params
        end
      end
    end 

    context 'as admin' do
      let!(:admin) {create(:administrator)}
      let!(:superadmin) { create(:user, roles: ['Superadmin'])}

      path '/api/dmo/entities' do
        post 'Create entity' do
          tags TAGS_API_DMO_ENTITIES
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]
          json_params = {}
          MUTATE_PARAMS.each do |key, val|
            json_params[key] = {type: nice_type(val)}
          end
          json_params[:lessor_id] = {type: :string}
          json_params[:housing_offer_ids] = {type: :array}
          json_params[:applicant_ids] = {type: :array}
          parameter name: :params, in: :body, schema: jsonify_params(json_params)
          #jsonapi_params( **json_params )

          response '201', 'entity created' do
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:params) do
              jsonapi_attributes(
              {}
              )
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'entities'
            end
          end
        end
      end

      path '/api/dmo/entities' do
        get 'Retrieves all entities' do
          tags TAGS_API_DMO_ENTITIES
          produces 'application/json'

          parameter name: 'ids[]', in: :query, type: :array, items: { type: :string }, collectionFormat: :multi, required: false
          ACCEPTABLE_PARAMS.each do |acceptable_param|
            type=''
            if MUTATE_PARAMS.include?(acceptable_param)
              type = nice_type(MUTATE_PARAMS[acceptable_param])
            end
            parameter name: acceptable_param.to_s, in: :query, type: type, required: false
          end
          # VALID_OPERATORS.each do |valid_operator|
          #   parameter( name: valid_operator.to_s, in: :query,type: :object, required: false)
          #end

          security [Bearer: {}]
          paginator

          response '200', 'entities found' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let!(:entities) {create_list(:entity, 5)}

            include_context 'with integration test'
            include_context 'valid jsonapi collection' do
              let(:expected_collection) {entities}
            end
          end
        end
      end

      path '/api/dmo/entities/{id}' do
        get 'Retrieves a single entity by id' do
          tags TAGS_API_DMO_ENTITIES
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '200', 'entity found' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:entity) {create(:entity)}
            let(:id) {entity.id}

            include_context 'with integration test'
            include_context 'valid jsonapi ressource' do
              let(:expected_ressource) {entity}
            end
          end
        end
      end

      path '/api/dmo/entities/{id}' do
        put 'edit entity' do
          tags TAGS_API_DMO_ENTITIES
          produces 'application/json'
          consumes 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]
          json_params = {}
          MUTATE_PARAMS.each do |key, val|
            json_params[key] = {type: nice_type(val)}
          end
          json_params[:lessor_id] = {type: :string}
          json_params[:housing_offer_ids] = {type: :array}
          json_params[:applicant_ids] = {type: :array}
          parameter name: :params, in: :body, schema: jsonify_params(json_params)
          #jsonapi_params( **json_params )

          response '200', 'entity updated' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:entity) {create(:entity)}
            let(:id) {entity.id}
            let(:params) do
              jsonapi_attributes(
               {title: 'String',}
              )
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'entities'
            end
          end
        end
      end

      path '/api/dmo/entities/{id}/switch_manual_mode' do
        put 'switch manual mode' do
          tags TAGS_API_DMO_ENTITIES
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]

          parameter name: :id, in: :path, type: :string

          response '200', 'housing_objectives found' do
            let(:Authorization) {"Bearer #{token_for(superadmin)}"}
            let(:entity) {create(:entity)}
            let(:id) {entity.id}

            include_context 'not containings errors'
            include_context 'with integration test' 
            after do |_example|
              json = JSON.parse(response.body)
              expect(json['code']).to eq(200)
            end
          end
        end
      end

      path '/api/dmo/entities/{id}' do
        delete 'Delete a single entity by id' do
          tags TAGS_API_DMO_ENTITIES
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '204', 'entity deleted' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:id) {create(:entity, responsible: admin.id).id}
            run_test!
          end
        end
      end
    end
  end
end
