require 'rails_helper'

RSpec.describe Hermes::HousingApplication, type: :model do
  describe 'edit with change status' do
    let!(:applicant) { create(:applicant) }
    let!(:applicant2) { create(:applicant) }
    let!(:housing_offer1) do
      create(
        :housing_offer,
        typology: 'T2',
        external_ref: 'blabli'
      )
    end

    let!(:housing_request1) do
      create(
        :housing_request,
        is_active: true,
        main_applicant: applicant
      )
    end

    let!(:housing_request2) do
      create(
        :housing_request,
        is_active: true,
        main_applicant: applicant2
      )
    end

    let!(:housing_application1) do
      create(
        :housing_application,
        housing_offer_id: housing_offer1.id,
        housing_request_id: housing_request1.id,
        status: 'TRANSMIS BAILLEUR'
      )
    end
    let!(:housing_application2) do
      create(
        :housing_application,
        housing_offer_id: housing_offer1.id,
        housing_request_id: housing_request1.id,
        status: 'TRANSMIS BAILLEUR'
      )
    end

    let!(:housing_application3) do
      create(
        :housing_application,
        housing_offer_id: housing_offer1.id,
        housing_request_id: housing_request2.id,
        status: 'TRANSMIS BAILLEUR'
      )
    end

    it 'triggers sopra flux' do
      allow(housing_application1).to receive(:update_status).and_call_original
      allow(Hermes::BoClient::UpdateStatusTask).to receive_message_chain(:new, :execute)
      housing_application1.update(status: 'ACCEPTEE EN CAL')

      expect(housing_application1).to have_received(:update_status).once
      expect(Hermes::BoClient::UpdateStatusTask.new).to have_received(:execute).once
    end

    it 'only 1 application active' do
      expect(housing_application1.reload.is_active).to eq(false)
      expect(housing_application2.reload.is_active).to eq(true)
      expect(housing_application1.reload.housing_request.id).to eq(housing_request1.id)
      expect(housing_application2.reload.housing_request.id).to eq(housing_request1.id)
      expect(housing_request1.reload.active_application_id).to eq(housing_application2.id.to_s)
      expect(housing_request1.reload.housing_application_ids.length).to eq(2)
      expect(housing_offer1.reload.applicated_nb).to eq(2)
    end
    it 'increment is visible in housing_request_serializer' do
       json = Hermes::HousingRequestSerializer.new(
        housing_request1.reload,
       # scope: scope,
        root: false
      ).to_json
      expect(JSON.parse(json)['applications_nb']).to eq(2)
    end
  end

  describe 'Statues' do
    let(:applicant) { create(:applicant) }
    let(:housing_offer1) { create(:housing_offer, offer_status: 'LOGEMENT PUBLIE', typology: 'T1') }
    let(:housing_request) { create(:housing_request, is_active: true, main_applicant: applicant) }
    let(:score1) { create(:score, housing_offer_id: housing_offer1.id.to_s, applicant_id: applicant.id.to_s) }
    let(:housing_application1) do
      create(
        :housing_application,
        housing_offer_id: housing_offer1.id,
        housing_request_id: housing_request.id,
        status: 'TRANSMIS BAILLEUR'
      )
    end

    it 'trigger acceptation' do
      housing_application1.update!(status: 'ACCEPTEE EN CAL', visit_status: 'VISITE EFFECTUEE')
      applicant.reload
      expect(applicant.completed_requests_nb).to eq(1)
      expect(score1.already_accepted_in_cal).to eq(true)
      jobs = enqueued_jobs.select {|job| job[:job] == UpdateScoreJob}
      expect(jobs).to include({:job=>UpdateScoreJob, :args=>["housing_request", [housing_offer1.id.to_s], applicant.id.to_s], :queue=>"low"})
    end
  end

  describe 'Cancelation' do
    let!(:canceler_applicant) { create(:applicant) }
    let!(:housing_offer1) { create(:housing_offer, offer_status: 'LOGEMENT PUBLIE', typology: 'T1') }
    let!(:housing_offer2) { create(:housing_offer, offer_status: 'LOGEMENT PUBLIE', typology: 'T1') }
    let!(:housing_request) { create(:housing_request, is_active: true, main_applicant: canceler_applicant) }
    let(:housing_application1) do
      create(
        :housing_application,
        housing_offer_id: housing_offer1.id,
        housing_request_id: housing_request.id,
        status: 'TRANSMIS BAILLEUR'
      )
    end
    let(:housing_application2) do
      create(
        :housing_application,
        housing_offer_id: housing_offer2.id,
        housing_request_id: housing_request.id,
        status: 'TRANSMIS BAILLEUR'
      )
    end

    before do
      drain_all_jobs
      [housing_application1, housing_application2].each(&:reload)
    end

    it 'trigger cancelation' do
      housing_application1.update!(status: 'DESISTEMENT', visit_status: 'VISITE EFFECTUEE')
      housing_application2.update!(status: 'DESISTEMENT', visit_status: 'VISITE EFFECTUEE')
      [housing_application1, housing_application2].each do |housing_app|
        housing_app.reload
        expect(housing_app.is_active).to be(false)
      end

      expect(canceler_applicant.reload.canceled_application_nb).to eq(2)
      expect(housing_request.reload.applications_nb).to eq(2)
      expect(housing_request.reload.active_application_id).to eq(nil)

      score_job = enqueued_jobs.find { |job| job[:job] == UpdateScoreJob }
      expect(score_job[:job]).to eq(UpdateScoreJob)
      expect(score_job[:args]).to eq(['housing_request', [housing_offer1.id.to_s, housing_offer2.id.to_s], canceler_applicant.id.to_s])
    end
  end

  describe 'Hooks' do
    let!(:lessor_user) {create(:user, first_name: 'Jaques', last_name: 'Dupont', email: 'jaques.dupont@gmail.com', roles: ['bailleur'])}
    let!(:lessor) {create(:lessor, user_id: lessor_user.id.to_s)}
    let!(:housing_offer) {create(:housing_offer, lessor: lessor)}
    let!(:applicant) { create(:applicant) }
    let!(:applicant2) { create(:applicant) }
    let!(:housing_request) {create(:housing_request, main_applicant_id: applicant.id, is_active: true, applicant_ids: [applicant2.id])}
    let!(:housing_application) {create(:housing_application, housing_request: housing_request, housing_offer: housing_offer)}

    before do
      applicant.reload
      applicant2.reload
    end

    it 'setup lessor as readers' do
      expect(housing_application.readers).to include(lessor_user.id.to_s)
      expect(housing_request.readers).to include(lessor_user.id.to_s)
      expect(applicant.readers).to include(lessor_user.id.to_s)
      expect(applicant2.readers).to include(lessor_user.id.to_s)
    end

    it 'remove lessor as readers' do
      housing_application.update!(status: 'ANNULEE BAILLEUR')
      applicant.reload
      applicant2.reload
      expect(housing_application.readers).not_to include(lessor_user.id.to_s)
      expect(housing_request.readers).not_to include(lessor_user.id.to_s)
      expect(applicant.readers).not_to include(lessor_user.id.to_s)
      expect(applicant2.readers).not_to include(lessor_user.id.to_s)
    end
  end
end
